@@ -1,5 +1,5 @@
 // Program     : x10-amh
-// Version     : 1.03
+// Version     : 1.05
 // Description : X10 Model CP-290 control program (available at Radio Shack)
 //               (computer control for X10 modules)
 //               This application allows you to control lights and appliances
@@ -44,7 +44,7 @@
 #define X10_MODE_TOMORROW 2
 #define X10_MODE_CLEAR    0
 
-#define VERSION         "v1.03"
+#define VERSION         "v1.05"
 
 /*
   Write n bytes from file descriptor, and don't return until done
@@ -55,8 +55,8 @@ static int
 write_n ( int fd, u_char *buf, int len )
 {
   for ( int num=0; num<len; num++ ) {
-	// Inter-character delay, because the CP-290 uses a software UART and
-	// would drop a character while it is performing other processing.
+        // Inter-character delay, because the CP-290 uses a software UART and
+        // would drop a character while it is performing other processing.
     usleep(30);
 
     if ( write( fd, &buf[num], 1) != 1 ) {
@@ -228,8 +228,8 @@ x10::msg( u_char *msg, int len )
   if( ! num_retries ) printf( "Error sending sync\n" );
 
   for( num_retries = 4; num_retries; num_retries-- ) {
-	// Flush characters waiting to be read - they are from a previous
-	// operation or a button-press on the CP-290.
+        // Flush characters waiting to be read - they are from a previous
+        // operation or a button-press on the CP-290.
     tcflush(this->fd, TCIFLUSH);
 
     if( write_n( this->fd, msg, len ) == len ) break;
@@ -447,28 +447,26 @@ usage( char *name )
 {
   printf( "x10 "VERSION" control program "
          "by Aaron Hightower (aaron@paradigmsim.com)\n"
-         "\nUsage:\n"
+         "=== Usage ===\n"
          "%s: [-v] [-c housecode] [-n list] [-f list] [-d dimlevel,list]\n\n"
          "-v : verbose , -t : self-test, -q : query CP-290's day and time\n"
          "-s : set CP-290's time according to CPU's day and time\n"
-         "-z [a-p] : set the x10's default base housecode\n"
+         "-z [a-p] : set the rocker button housecode for the CP290\n"
          "-c [a-p] : use alternate house code (default \"a\")\n"
          "-n list : turn oN devices in list\n"
          "-f list : turn oFf devices in list\n"
+         "-e : empty conents of CP-290's internal events\n"
          "-l : list contents of CP-290 to stdout\n"
          "-d dimlevel,list : dim devices in list to dimlevel\n"
          "list is a comma separated list of devices, each ranging from 1 to 16"
          "\ndimlevel is an integer from 0 to 15 (0 brightest)\n"
          "\nExamples:\n"
-         "\tTurn on devices a1 a2 a3\n"
-         "\t%s -n 1,2,3\n\n"
-         "\tTurn off devices b1 b2 b3\n"
-         "\t%s -c b -f 4,6\n\n"
-         "\tdim lights 2 and 3 to 5th brightest setting\n"
-         "\t%s -d 5,2,3\n\n"
-		 "\tNote that all commands will use house code \"a\" unless you\n"
-		 "\tuse the \"-c\" argument. The default house code set by the \"-z\"\n"
-		 "\tcommand is only used by the keyboard buttons on the CP-290.\n\n"
+         "\tTurn on devices a1 a2 a3: \"%s -n 1,2,3 \"\n"
+         "\tTurn off devices b4 b6  : \"%s -c b -f 4,6\"\n"
+         "\tdim lights 2 and 3 to 5th brightest setting: \"%s -d 5,2,3\"\n\n"
+         "Note: All commands use house code \"a\" unless you use the \"-c\" "
+         "argument.\nThe house code set by the \"-z\" command is for the "
+         "buttons on the CP-290.\n"
           , name, name, name, name );
   exit( 0 );
 }
@@ -562,28 +560,28 @@ x10::readtab( )
          ask=5;
 
   for ( j = 0; j < 6; j++ ) {
-	  // This seems to fail sometimes because of a read-buffer overrun.
-	  // flush the read buffer and start over, and hope I get all of the
-	  // characters this time.
-	  tcflush(this->fd, TCIOFLUSH);
-	  msg( &ask, 1   );
-	  if( -1 == read_n( this->fd, result, 7 ) )
-		continue;
+          // This seems to fail sometimes because of a read-buffer overrun.
+          // flush the read buffer and start over, and hope I get all of the
+          // characters this time.
+          tcflush(this->fd, TCIOFLUSH);
+          msg( &ask, 1   );
+          if( -1 == read_n( this->fd, result, 7 ) )
+                continue;
 
-	  for( i=0; i<6; i++ ) {
-			if( result[i] != 255 )
-				continue;
-	  }
-	  break;
-	  sleep(5);
+          for( i=0; i<6; i++ ) {
+                        if( result[i] != 255 )
+                                continue;
+          }
+          break;
+          sleep(5);
   }
   if ( j == 7 ) {
-	printf( "Did not receive sync from CP-290\n" );
-	return;
+        fprintf( stderr, "Did not receive sync from CP-290\n" );
+        return;
   }
   if( result[6] == 0 ) {
 
-    printf( "Warning!  CP-290 has been reset.  All memory has been lost\n" );
+    fprintf( stderr, "Warning!  CP-290 has been reset.  All memory has been lost\n" );
     listvalid = 1;
     return;
   }
@@ -595,14 +593,24 @@ x10::readtab( )
 
     if( events[i][0] != 255 ) {
       read_n( this->fd, & events[i][1], 7 );
-      printf( "\n" );
+      fprintf( stderr, "\n" );
     }
     else fprintf( stderr, "empty\n" );
   }
 
   listvalid = 1;
 
-  printf( "\n" );
+  fprintf( stderr, "\n" );
+}
+
+void
+x10::emptytab( void )
+{
+  fprintf( stderr, "Erasing all of CP-290's internal memory\n" );
+
+  bzero( events, sizeof( events ) );
+
+  writetab( 1 );
 }
 
 void
@@ -676,7 +684,6 @@ x10::printevent( int i )
 
     switch( housecode ) {
 
-      case 0x6: housecode = 'a'; break;
       case 0xe: housecode = 'b'; break;
       case 0x2: housecode = 'c'; break;
       case 0xa: housecode = 'd'; break;
@@ -692,18 +699,20 @@ x10::printevent( int i )
       case 0x8: housecode = 'n'; break;
       case 0x4: housecode = 'o'; break;
       case 0xc: housecode = 'p'; break;
-      default:  housecode = 'a'; break;
+      case 0x6: // House code 'a' doesn't print since it's the default
+      default:  housecode =  0 ; break;
     }
 
-    printf( "event {\n" );
-    printf( "  devmap    %s\n", bmtostr( devbm ) );
-    printf( "  daymap    %s\n", bmtostr( daymap << 9 ) );
-    printf( "  function  %s\n", funcdesc );
-    printf( "  hour      %d\n", hour );
-    printf( "  minute    %d\n", minute );
-    printf( "  mode      %s\n", modedesc );
-    printf( "  housecode %c\n", housecode );
-    printf( "  dimlevel  %d\n", level );
+                    printf( "event {\n" );
+                    printf( "  devmap    %s\n", bmtostr( devbm ) );
+                    printf( "  daymap    %s\n", bmtostr( daymap << 9 ) );
+                    printf( "  function  %s\n", funcdesc );
+                    printf( "  hour      %d\n", hour );
+    if( minute    ) printf( "  minute    %d\n", minute );
+                    printf( "  mode      %s\n", modedesc );
+    if( housecode ) printf( "  housecode %c\n", housecode );
+    if( level     ) printf( "  dimlevel  %d\n", level );
+
     printf( "}\n\n" );
   }
 }
@@ -812,7 +821,7 @@ x10::readtab( const char *filename )
 
         if( strcmp( word, "{" ) ) { /* } */
 
-	  printf( "Expecting an open brace" );
+          fprintf( stderr, "Expecting an open brace" );
         }
       }
       else if( !strcmp( word, "housecode" ) ) housecode = strdup( word = strtok( NULL, " \t\n" ) );
@@ -828,7 +837,8 @@ x10::readtab( const char *filename )
 
         int intfunc;
 
-             if( !strcasecmp( func, "on"  ) ) intfunc = X10_ON;
+        if( !func ) intfunc = X10_ON;
+        else if( !strcasecmp( func, "on"  ) ) intfunc = X10_ON;
         else if( !strcasecmp( func, "off" ) ) intfunc = X10_OFF;
         else if( !strcasecmp( func, "dim" ) ) intfunc = X10_DIM;
         else intfunc = X10_ON;
@@ -837,42 +847,44 @@ x10::readtab( const char *filename )
 
         int intmode;
 
-             if( !strcasecmp( func, "normal"   ) ) intmode = X10_MODE_NORMAL;
-	else if( !strcasecmp( func, "security" ) ) intmode = X10_MODE_SECURITY;
-	else if( !strcasecmp( func, "today"    ) ) intmode = X10_MODE_TODAY;
-	else if( !strcasecmp( func, "tomorrow" ) ) intmode = X10_MODE_TOMORROW;
-	else if( !strcasecmp( func, "clear"    ) ) intmode = X10_MODE_CLEAR;
-	else intmode = X10_MODE_NORMAL;
+             if( !strcasecmp( mode, "normal"   ) ) intmode = X10_MODE_NORMAL;
+        else if( !strcasecmp( mode, "security" ) ) intmode = X10_MODE_SECURITY;
+        else if( !strcasecmp( mode, "today"    ) ) intmode = X10_MODE_TODAY;
+        else if( !strcasecmp( mode, "tomorrow" ) ) intmode = X10_MODE_TOMORROW;
+        else if( !strcasecmp( mode, "clear"    ) ) intmode = X10_MODE_CLEAR;
+        else intmode = X10_MODE_NORMAL;
 
-        printf( "housecode = %s\n", housecode );
-        printf( "devmap    = %s\n", devmap    );
-        printf( "daymap    = %s\n", daymap    );
-        printf( "mode      = %s\n", mode      );
-        printf( "minute    = %d\n", minute    );
-        printf( "hour      = %d\n", hour      );
-        printf( "function  = %s\n", func      );
+        if( housecode ) printf( "housecode = %s\n", housecode );
+        if( devmap    ) printf( "devmap    = %s\n", devmap    );
+        if( daymap    ) printf( "daymap    = %s\n", daymap    );
+        if( mode      ) printf( "mode      = %s\n", mode      );
+        if( minute    ) printf( "minute    = %d\n", minute    );
+        if( hour      ) printf( "hour      = %d\n", hour      );
+        if( func      ) printf( "function  = %s\n", func      );
 
         if( dimlevel )
           printf( "dimlevel  = %d\n", dimlevel  );
 
         printf( "\n" );
 
-        setEvent( i++, housecode[0], devmap, daymap, intmode, minute, hour, dimlevel, intfunc );
+        setEvent( i++, housecode ? housecode[0] : 'a',
+                  devmap, daymap, intmode, minute, hour,
+                  dimlevel, intfunc );
 
-	if( housecode ) free( housecode );
-	if( devmap    ) free( devmap    );
-	if( daymap    ) free( daymap    );
-	if( func      ) free( func      );
-	if( mode      ) free( mode      );
+        if( housecode ) free( housecode );
+        if( devmap    ) free( devmap    );
+        if( daymap    ) free( daymap    );
+        if( func      ) free( func      );
+        if( mode      ) free( mode      );
 
-	housecode = 0;
-	devmap    = 0;
-	daymap    = 0;
-	mode      = 0;
-	minute    = 0;
-	hour      = 0;
-	dimlevel  = 0;
-	func      = 0;
+        housecode = 0;
+        devmap    = 0;
+        daymap    = 0;
+        mode      = 0;
+        minute    = 0;
+        hour      = 0;
+        dimlevel  = 0;
+        func      = 0;
       }
 
       word = strtok( NULL, " \t\n" );
@@ -885,11 +897,11 @@ x10::setEvent( int i, char housecode, const char *devmap, const char *daymap,
                int mode, int minute, int hour, int dimlevel, int func )
 {
   events[i][0] = ( mode   & 0x0f );
-  events[i][1] = ( ( strtobm( daymap ) ) >> 9 );
+  events[i][1] = ( ( strtobm( daymap ? daymap : "1,2,3,4,5,6,7" ) ) >> 9 );
   events[i][2] = ( hour   & 0x1f );
   events[i][3] = ( minute & 0x3f );
-  events[i][4] = ( strtobm( devmap ) >> 8 ) & 0xff;
-  events[i][5] = ( strtobm( devmap ) >> 0 ) & 0xff;
+  events[i][4] = ( strtobm( devmap ? devmap : "1" ) >> 8 ) & 0xff;
+  events[i][5] = ( strtobm( devmap ? devmap : "1" ) >> 0 ) & 0xff;
   events[i][6] = chartohc( housecode );
   events[i][7] = ( ( dimlevel & 0xf ) << 4 ) | ( func & 0xf );
 }
@@ -923,7 +935,7 @@ main( int argc, char * const argv[] )
 
   int c;
 
-  while( ( c = getopt( argc, argv, "z:d:lvhc:n:f:qst" ) ) != -1 ) {
+  while( ( c = getopt( argc, argv, "z:d:elvhc:n:f:qst" ) ) != -1 ) {
 
     switch( c ) {
 
@@ -951,6 +963,10 @@ main( int argc, char * const argv[] )
         }
         break;
 
+      case 'e':
+        x->emptytab( );
+        break;
+
       case 'l':
         x->readtab( );
         x->listtab( );
